"use strict";
var CACHE_PREFIX = "__offline_cache_key::";
var LOG_PREFIX = "[OFFLINE CACHE]:";
function init() {
    (function (window) {
        var theRealFetch = window.fetch;
        window.fetch = function fetchWithOfflineReplays(input, init) {
            var requestUrl = "";
            var requestMethod = "GET";
            if (input instanceof Request) {
                requestUrl = input.url;
                requestMethod = input.method;
            }
            else {
                requestUrl = input;
                requestMethod = "GET";
            }
            var key = "" + CACHE_PREFIX + requestMethod + "::" + requestUrl;
            if (navigator.onLine === true) {
                return fetchAndCache(theRealFetch, input, init);
            }
            else {
                return attemptFetchFromCache(input, init);
            }
        };
    })(window);
}
function fetchAndCache(fetch, input, init) {
    return fetch(input, init).then(function (response) {
        if (!response.ok) {
            return response;
        }
        response
            .clone()
            .text()
            .then(function (text) {
            var value = JSON.stringify({
                status: response.status,
                statusText: response.statusText,
                body: text
            });
            var key = generateKey(input);
            localStorage.setItem(key, value);
        });
        return response;
    });
}
function attemptFetchFromCache(input, init) {
    var requestUrl = input instanceof Request ? input.url : input;
    console.log(LOG_PREFIX + " You are offline. Checking cache for " + requestUrl + "...");
    var key = generateKey(input);
    var value = localStorage.getItem(key);
    if (value === null) {
        return Promise.reject("There was an offline request made with no cached response.");
    }
    var cachedResponse = JSON.parse(value);
    var response = new Response(cachedResponse.body, {
        status: cachedResponse.status,
        statusText: cachedResponse.statusText
    });
    console.log(LOG_PREFIX + " Found a cached response with a body of length " + cachedResponse.body.length);
    return Promise.resolve(response);
}
function generateKey(input) {
    var requestUrl = input instanceof Request ? input.url : input;
    var requestMethod = input instanceof Request ? input.method : "GET";
    return "" + CACHE_PREFIX + requestMethod + "::" + requestUrl;
}
init();
//# sourceMappingURL=laurence.js.map